# Questions and Answers
Please feel free to add you questions and anwsers.

[[_TOC_]]


## Q: How to use ssh-key authentication?
 
In order to connect passwordless to a remote server a key needs to be generated.
```
$ ssh-keygen -b 4096
Generating public/private rsa key pair.
Enter passphrase (empty for no passphrase): 
Enter same passphrase again: 
Your identification has been saved in .ssh/id_rsa
Your public key has been saved in .ssh/id_rsa.pub
The key fingerprint is:
SHA256:lHEKWnP+1vdNfbSELXApKpFvllcgCp3DHsgrbO2RYuo mblaschek@pop-os
The key's randomart image is:
+---[RSA 4096]----+
|    ..B =..o...  |
|     =.&.=..ooo  |
|  . o +.X o oo o.|
|   * = + B o  o.o|
|  + + . S + . ..+|
| .   .   .   . oo|
|.               o|
| E               |
|                 |
+----[SHA256]-----+
```

It is recommended to use a password to encrpyt the private key `.ssh/id_rsa`. However, this password is then required each time to login. Using an  `ssh-agent` can solve that problem.

## Q: How to use an ssh-agent?
Using an SSH-Agent will make your connection even safer, as your private key is encrypted with a passphrase. To create a ssh-key goto [How to use ssh-key authentication?](#How-to-use-ssh-key-authentication?).

Continue with how to use the ssh-agent:
```bash
# Launch the SSH Agent
eval `ssh-agent`
# this will add your ssh-keys from ~/.ssh/
# You will be asked a passphrase if you have one
# this will add .ssh/id_rsa as default
ssh-add 
# or any keyfile you created:
ssh-add .ssh/secret_key
# Copy the ssh-key (pub) to the server 
ssh-copy-id [user]@[server]
# and connect
ssh [user]@[server]
# Kill the agent and all store secure information
ssh-agent -k
```
**Tip: configure your `.ssh/config` file to use the correct key a server. Goto [SSH](SSH.md)**

[Nice summary of how an ssh-agent works](https://smallstep.com/blog/ssh-agent-explained/)

## Q: How to transfer files between two VPN networks?

You should be able to use an SSH tunnel via a gateway server

Situation
```
  VPN-1                                              VPN-2
__________    /|       ___________      /|         __________
|  local |   | |       | gateway |     | |         | remote |
|        |---| |----<>------     |   Firewall      |        |
|        | Firewall    |    \---<>-----| |---------|        |
__________   | |       ___________     | |         __________
             |/                        |/
```
Assuming you're trying to transfer a file from/to a remote computer ("remote") from/to your local computer ("local"), 
establish a tunnel via a third computer ("gateway") by typing this on your local computer:

```bash
$ ssh -fNL 12345:remote:22 gatewaylogin@gateway
```
Then you can run an unlimited amount of SCP commands on this tunnel (still typing on your local computer):

```bash
# scp [SRC] [DEST]
$ scp -P 12345 remotelogin@localhost://path/to/remote/file /local/path/where/you/want/file
```

another option is to use `rsync` with a gateway command:
```bash
# rsync [SRC] [DEST]
$ rsync -avz -r --stats --progress -e "ssh gateway ssh" remote:/src/documents/ /dest/documents 
```
**Note: key-based authentication is required from the gateway to the remote server using rsync.**
Initial testing showed that using key-based authentication between gateway and remote is required.

## Q: How to connect to Jet, SRVX8, SRVX2?

Currently (01.2021) only `srvx1.img.univie.ac.at` and `login.univie.ac.at` are available from the internet.
Please replace `[USER]` with your username on these servers and adjust the servers to your needs.

```bash
$ ssh -t [USER]@srvx1.img.univie.ac.at 'ssh [USER]@jet01.img.univie.ac.at'
```
or using the above `~/.ssh/config` you can do:
```bash
$ ssh -t login ssh jet
```
or using the connect script
```bash
$ connect2jet -g login jet
```

## Q: How to mount a remote file system on Linux (MAC)?
You can us programs like [Filezilla](https://filezilla-project.org/) or [Cyberduck](https://cyberduck.io/) (MAC) to transfer files between remote and local host. But sometimes it is much easier to mount a remote file system and work on it like an external drive.
You can use the [mountserver](mountserver) script to do so. This requires `sshfs` to be installed, on Linux that is in the standard repositories. A short into and some additional steps can be found [here](https://www.tjansson.dk/2008/01/autofs-and-sshfs-the-perfect-couple/).

```bash
$ mountserver [remote] [remote directory] [local directory]
# example
$ mountserver srvx1 /raid61/wetter /home/monkey/srvx1
```

This will mount the remote directory to the local directory. The local directory will be created if it does not exist. The directory should be empty before mounting, otherwise that will cause problems.



